@@ -19,12 +19,11 @@ package v1beta1
 import (
 	"context"
 	"testing"
-	"time"
 
 	"github.com/google/go-cmp/cmp"
 	"github.com/google/go-cmp/cmp/cmpopts"
 	"github.com/tektoncd/pipeline/test/diff"
-	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/util/sets"
 	"knative.dev/pkg/apis"
 )
@@ -139,6 +138,37 @@ func TestPipelineTask_ValidateCustomTask(t *testing.T) {
 			Message: `invalid value: custom task ref must specify kind`,
 			Paths:   []string{"taskRef.kind"},
 		},
+	}, {
+		name: "custom task - taskSpec without kind",
+		task: PipelineTask{Name: "foo", TaskSpec: &EmbeddedTask{
+			TypeMeta: runtime.TypeMeta{
+				APIVersion: "example.dev/v0",
+				Kind:       "",
+			},
+		}},
+		expectedError: apis.FieldError{
+			Message: `invalid value: custom task spec must specify kind`,
+			Paths:   []string{"taskSpec.kind"},
+		},
+	}, {
+		name: "custom task - taskSpec without apiVersion",
+		task: PipelineTask{Name: "foo", TaskSpec: &EmbeddedTask{
+			TypeMeta: runtime.TypeMeta{
+				APIVersion: "",
+				Kind:       "some-kind",
+			},
+		}},
+		expectedError: apis.FieldError{
+			Message: `invalid value: custom task spec must specify apiVersion`,
+			Paths:   []string{"taskSpec.apiVersion"},
+		},
+	}, {
+		name: "custom task - taskRef without apiVersion",
+		task: PipelineTask{Name: "foo", TaskRef: &TaskRef{APIVersion: "", Kind: "some-kind", Name: ""}},
+		expectedError: apis.FieldError{
+			Message: `invalid value: custom task ref must specify apiVersion`,
+			Paths:   []string{"taskRef.apiVersion"},
+		},
 	}, {
 		name: "custom task doesn't support conditions",
 		task: PipelineTask{
@@ -152,17 +182,6 @@ func TestPipelineTask_ValidateCustomTask(t *testing.T) {
 			Message: `invalid value: custom tasks do not support conditions - use when expressions instead`,
 			Paths:   []string{"conditions"},
 		},
-	}, {
-		name: "custom task doesn't support retries",
-		task: PipelineTask{
-			Name:    "foo",
-			Retries: 3,
-			TaskRef: &TaskRef{APIVersion: "example.dev/v0", Kind: "Example"},
-		},
-		expectedError: apis.FieldError{
-			Message: `invalid value: custom tasks do not support retries`,
-			Paths:   []string{"retries"},
-		},
 	}, {
 		name: "custom task doesn't support pipeline resources",
 		task: PipelineTask{
@@ -174,17 +193,6 @@ func TestPipelineTask_ValidateCustomTask(t *testing.T) {
 			Message: `invalid value: custom tasks do not support PipelineResources`,
 			Paths:   []string{"resources"},
 		},
-	}, {
-		name: "custom task doesn't support timeout",
-		task: PipelineTask{
-			Name:    "foo",
-			Timeout: &metav1.Duration{Duration: time.Duration(3)},
-			TaskRef: &TaskRef{APIVersion: "example.dev/v0", Kind: "Example"},
-		},
-		expectedError: apis.FieldError{
-			Message: `invalid value: custom tasks do not support timeout`,
-			Paths:   []string{"timeout"},
-		},
 	}}
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
