@@ -163,21 +163,21 @@ touch ${scriptfile} && chmod +x ${scriptfile}
 cat > ${scriptfile} << '_EOF_'
 IyEvYmluL3NoCnNjcmlwdC0x
 _EOF_
-/tekton/tools/entrypoint decode-script "${scriptfile}"
+/tekton/bin/entrypoint decode-script "${scriptfile}"
 scriptfile="/tekton/scripts/script-2-mz4c7"
 touch ${scriptfile} && chmod +x ${scriptfile}
 cat > ${scriptfile} << '_EOF_'
 CiMhL2Jpbi9zaApzY3JpcHQtMw==
 _EOF_
-/tekton/tools/entrypoint decode-script "${scriptfile}"
+/tekton/bin/entrypoint decode-script "${scriptfile}"
 scriptfile="/tekton/scripts/script-3-mssqb"
 touch ${scriptfile} && chmod +x ${scriptfile}
 cat > ${scriptfile} << '_EOF_'
-IyEvYmluL3NoCnNldCAteGUKbm8tc2hlYmFuZw==
+IyEvYmluL3NoCnNldCAtZQpuby1zaGViYW5n
 _EOF_
-/tekton/tools/entrypoint decode-script "${scriptfile}"
+/tekton/bin/entrypoint decode-script "${scriptfile}"
 `},
-		VolumeMounts: []corev1.VolumeMount{writeScriptsVolumeMount, toolsMount},
+		VolumeMounts: []corev1.VolumeMount{writeScriptsVolumeMount, binMount},
 	}
 	want := []corev1.Container{{
 		Image:        "step-1",
@@ -247,7 +247,7 @@ script-3`,
 			Args:         []string{"my", "args"},
 		},
 	}}, []v1beta1.Sidecar{}, &v1beta1.TaskRunDebug{
-		Breakpoint: []string{BreakpointOnFailure},
+		Breakpoint: []string{breakpointOnFailure},
 	})
 	wantInit := &corev1.Container{
 		Name:    "place-scripts",
@@ -258,35 +258,35 @@ touch ${scriptfile} && chmod +x ${scriptfile}
 cat > ${scriptfile} << '_EOF_'
 IyEvYmluL3NoCnNjcmlwdC0x
 _EOF_
-/tekton/tools/entrypoint decode-script "${scriptfile}"
+/tekton/bin/entrypoint decode-script "${scriptfile}"
 scriptfile="/tekton/scripts/script-2-mz4c7"
 touch ${scriptfile} && chmod +x ${scriptfile}
 cat > ${scriptfile} << '_EOF_'
 CiMhL2Jpbi9zaApzY3JpcHQtMw==
 _EOF_
-/tekton/tools/entrypoint decode-script "${scriptfile}"
+/tekton/bin/entrypoint decode-script "${scriptfile}"
 scriptfile="/tekton/scripts/script-3-mssqb"
 touch ${scriptfile} && chmod +x ${scriptfile}
 cat > ${scriptfile} << '_EOF_'
-IyEvYmluL3NoCnNldCAteGUKbm8tc2hlYmFuZw==
+IyEvYmluL3NoCnNldCAtZQpuby1zaGViYW5n
 _EOF_
-/tekton/tools/entrypoint decode-script "${scriptfile}"
+/tekton/bin/entrypoint decode-script "${scriptfile}"
 tmpfile="/tekton/debug/scripts/debug-continue"
 touch ${tmpfile} && chmod +x ${tmpfile}
 cat > ${tmpfile} << 'debug-continue-heredoc-randomly-generated-78c5n'
 #!/bin/sh
-set -xe
+set -e
 
 numberOfSteps=4
 debugInfo=/tekton/debug/info
-tektonTools=/tekton/tools
+tektonRun=/tekton/run
 
 postFile="$(ls ${debugInfo} | grep -E '[0-9]+' | tail -1)"
 stepNumber="$(echo ${postFile} | sed 's/[^0-9]*//g')"
 
 if [ $stepNumber -lt $numberOfSteps ]; then
-	touch ${tektonTools}/${stepNumber} # Mark step as success
-	echo "0" > ${tektonTools}/${stepNumber}.breakpointexit
+	touch ${tektonRun}/${stepNumber}/out # Mark step as success
+	echo "0" > ${tektonRun}/${stepNumber}/out.breakpointexit
 	echo "Executing step $stepNumber..."
 else
 	echo "Last step (no. $stepNumber) has already been executed, breakpoint exiting !"
@@ -297,18 +297,18 @@ tmpfile="/tekton/debug/scripts/debug-fail-continue"
 touch ${tmpfile} && chmod +x ${tmpfile}
 cat > ${tmpfile} << 'debug-fail-continue-heredoc-randomly-generated-6nl7g'
 #!/bin/sh
-set -xe
+set -e
 
 numberOfSteps=4
 debugInfo=/tekton/debug/info
-tektonTools=/tekton/tools
+tektonRun=/tekton/run
 
 postFile="$(ls ${debugInfo} | grep -E '[0-9]+' | tail -1)"
 stepNumber="$(echo ${postFile} | sed 's/[^0-9]*//g')"
 
 if [ $stepNumber -lt $numberOfSteps ]; then
-	touch ${tektonTools}/${stepNumber}.err # Mark step as a failure
-	echo "1" > ${tektonTools}/${stepNumber}.breakpointexit
+	touch ${tektonRun}/${stepNumber}/out.err # Mark step as a failure
+	echo "1" > ${tektonRun}/${stepNumber}/out.breakpointexit
 	echo "Executing step $stepNumber..."
 else
 	echo "Last step (no. $stepNumber) has already been executed, breakpoint exiting !"
@@ -316,7 +316,7 @@ else
 fi
 debug-fail-continue-heredoc-randomly-generated-6nl7g
 `},
-		VolumeMounts: []corev1.VolumeMount{writeScriptsVolumeMount, toolsMount, debugScriptsVolumeMount},
+		VolumeMounts: []corev1.VolumeMount{writeScriptsVolumeMount, binMount, debugScriptsVolumeMount},
 	}
 	want := []corev1.Container{{
 		Image:   "step-1",
@@ -394,21 +394,21 @@ touch ${scriptfile} && chmod +x ${scriptfile}
 cat > ${scriptfile} << '_EOF_'
 IyEvYmluL3NoCnNjcmlwdC0x
 _EOF_
-/tekton/tools/entrypoint decode-script "${scriptfile}"
+/tekton/bin/entrypoint decode-script "${scriptfile}"
 scriptfile="/tekton/scripts/script-2-mz4c7"
 touch ${scriptfile} && chmod +x ${scriptfile}
 cat > ${scriptfile} << '_EOF_'
 IyEvYmluL3NoCnNjcmlwdC0z
 _EOF_
-/tekton/tools/entrypoint decode-script "${scriptfile}"
+/tekton/bin/entrypoint decode-script "${scriptfile}"
 scriptfile="/tekton/scripts/sidecar-script-0-mssqb"
 touch ${scriptfile} && chmod +x ${scriptfile}
 cat > ${scriptfile} << '_EOF_'
 IyEvYmluL3NoCnNpZGVjYXItMQ==
 _EOF_
-/tekton/tools/entrypoint decode-script "${scriptfile}"
+/tekton/bin/entrypoint decode-script "${scriptfile}"
 `},
-		VolumeMounts: []corev1.VolumeMount{writeScriptsVolumeMount, toolsMount},
+		VolumeMounts: []corev1.VolumeMount{writeScriptsVolumeMount, binMount},
 	}
 	want := []corev1.Container{{
 		Image:        "step-1",
@@ -499,7 +499,7 @@ script-3
 no-shebang
 "@ | Out-File -FilePath /tekton/scripts/script-3-mssqb.cmd
 `},
-		VolumeMounts: []corev1.VolumeMount{writeScriptsVolumeMount, toolsMount},
+		VolumeMounts: []corev1.VolumeMount{writeScriptsVolumeMount, binMount},
 	}
 	want := []corev1.Container{{
 		Image:        "step-1",
@@ -583,7 +583,7 @@ script-3
 sidecar-1
 "@ | Out-File -FilePath /tekton/scripts/sidecar-script-0-mssqb
 `},
-		VolumeMounts: []corev1.VolumeMount{writeScriptsVolumeMount, toolsMount},
+		VolumeMounts: []corev1.VolumeMount{writeScriptsVolumeMount, binMount},
 	}
 	want := []corev1.Container{{
 		Image:        "step-1",
@@ -645,7 +645,7 @@ sidecar-1`,
 sidecar-1
 "@ | Out-File -FilePath /tekton/scripts/sidecar-script-0-9l9zj
 `},
-		VolumeMounts: []corev1.VolumeMount{writeScriptsVolumeMount, toolsMount},
+		VolumeMounts: []corev1.VolumeMount{writeScriptsVolumeMount, binMount},
 	}
 	want := []corev1.Container{{
 		Image: "step-1",
