@@ -32,6 +32,8 @@ import (
 	"github.com/tektoncd/pipeline/test/names"
 	corev1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"knative.dev/pkg/apis"
+	duckv1beta1 "knative.dev/pkg/apis/duck/v1beta1"
 )
 
 var (
@@ -40,7 +42,7 @@ var (
 		NopImage:                 "override-with-nop:latest",
 		GitImage:                 "override-with-git:latest",
 		KubeconfigWriterImage:    "override-with-kubeconfig-writer-image:latest",
-		ShellImage:               "mirror.gcr.io/library/busybox",
+		ShellImage:               "busybox",
 		GsutilImage:              "gcr.io/google.com/cloudsdktool/cloud-sdk",
 		PRImage:                  "override-with-pr:latest",
 		ImageDigestExporterImage: "override-with-imagedigest-exporter-image:latest",
@@ -50,22 +52,22 @@ var (
 		Sidecars: []v1beta1.Sidecar{{
 			Container: corev1.Container{
 				Name:  "foo",
-				Image: "$(inputs.params.myimage)",
+				Image: `$(params["myimage"])`,
 				Env: []corev1.EnvVar{{
 					Name:  "foo",
-					Value: "$(inputs.params.FOO)",
+					Value: "$(params['FOO'])",
 				}},
 			},
 		}},
 		StepTemplate: &corev1.Container{
 			Env: []corev1.EnvVar{{
 				Name:  "template-var",
-				Value: "$(inputs.params.FOO)",
+				Value: `$(params["FOO"])`,
 			}},
 		},
 		Steps: []v1beta1.Step{{Container: corev1.Container{
 			Name:  "foo",
-			Image: "$(inputs.params.myimage)",
+			Image: "$(params.myimage)",
 		}}, {Container: corev1.Container{
 			Name:       "baz",
 			Image:      "bat",
@@ -73,14 +75,14 @@ var (
 			Args:       []string{"$(inputs.resources.workspace.url)"},
 		}}, {Container: corev1.Container{
 			Name:  "qux",
-			Image: "$(inputs.params.something)",
+			Image: "$(params.something)",
 			Args:  []string{"$(outputs.resources.imageToUse.url)"},
 		}}, {Container: corev1.Container{
 			Name:  "foo",
-			Image: "$(inputs.params.myimage)",
+			Image: `$(params["myimage"])`,
 		}}, {Container: corev1.Container{
 			Name:       "baz",
-			Image:      "$(inputs.params.somethingelse)",
+			Image:      "$(params.somethingelse)",
 			WorkingDir: "$(inputs.resources.workspace.path)",
 			Args:       []string{"$(inputs.resources.workspace.url)"},
 		}}, {Container: corev1.Container{
@@ -89,44 +91,44 @@ var (
 			Args:  []string{"$(outputs.resources.imageToUse.url)"},
 		}}, {Container: corev1.Container{
 			Name:  "foo",
-			Image: "busybox:$(inputs.params.FOO)",
+			Image: "busybox:$(params.FOO)",
 			VolumeMounts: []corev1.VolumeMount{{
-				Name:      "$(inputs.params.FOO)",
-				MountPath: "path/to/$(inputs.params.FOO)",
-				SubPath:   "sub/$(inputs.params.FOO)/path",
+				Name:      "$(params.FOO)",
+				MountPath: "path/to/$(params.FOO)",
+				SubPath:   "sub/$(params.FOO)/path",
 			}},
 		}}, {Container: corev1.Container{
 			Name:  "foo",
-			Image: "busybox:$(inputs.params.FOO)",
+			Image: "busybox:$(params.FOO)",
 			Env: []corev1.EnvVar{{
 				Name:  "foo",
-				Value: "value-$(inputs.params.FOO)",
+				Value: "value-$(params.FOO)",
 			}, {
 				Name: "bar",
 				ValueFrom: &corev1.EnvVarSource{
 					ConfigMapKeyRef: &corev1.ConfigMapKeySelector{
-						LocalObjectReference: corev1.LocalObjectReference{Name: "config-$(inputs.params.FOO)"},
-						Key:                  "config-key-$(inputs.params.FOO)",
+						LocalObjectReference: corev1.LocalObjectReference{Name: "config-$(params.FOO)"},
+						Key:                  "config-key-$(params.FOO)",
 					},
 				},
 			}, {
 				Name: "baz",
 				ValueFrom: &corev1.EnvVarSource{
 					SecretKeyRef: &corev1.SecretKeySelector{
-						LocalObjectReference: corev1.LocalObjectReference{Name: "secret-$(inputs.params.FOO)"},
-						Key:                  "secret-key-$(inputs.params.FOO)",
+						LocalObjectReference: corev1.LocalObjectReference{Name: "secret-$(params.FOO)"},
+						Key:                  "secret-key-$(params.FOO)",
 					},
 				},
 			}},
 			EnvFrom: []corev1.EnvFromSource{{
-				Prefix: "prefix-0-$(inputs.params.FOO)",
+				Prefix: "prefix-0-$(params.FOO)",
 				ConfigMapRef: &corev1.ConfigMapEnvSource{
-					LocalObjectReference: corev1.LocalObjectReference{Name: "config-$(inputs.params.FOO)"},
+					LocalObjectReference: corev1.LocalObjectReference{Name: "config-$(params.FOO)"},
 				},
 			}, {
-				Prefix: "prefix-1-$(inputs.params.FOO)",
+				Prefix: "prefix-1-$(params.FOO)",
 				SecretRef: &corev1.SecretEnvSource{
-					LocalObjectReference: corev1.LocalObjectReference{Name: "secret-$(inputs.params.FOO)"},
+					LocalObjectReference: corev1.LocalObjectReference{Name: "secret-$(params.FOO)"},
 				},
 			}},
 		}}, {Container: corev1.Container{
@@ -137,15 +139,15 @@ var (
 			Image: "$(outputs.resources.imageToUse-re.path)",
 		}}},
 		Volumes: []corev1.Volume{{
-			Name: "$(inputs.params.FOO)",
+			Name: "$(params.FOO)",
 			VolumeSource: corev1.VolumeSource{
 				ConfigMap: &corev1.ConfigMapVolumeSource{
 					LocalObjectReference: corev1.LocalObjectReference{
-						Name: "$(inputs.params.FOO)",
+						Name: "$(params.FOO)",
 					},
 					Items: []corev1.KeyToPath{{
-						Key:  "$(inputs.params.FOO)",
-						Path: "$(inputs.params.FOO)",
+						Key:  "$(params.FOO)",
+						Path: "$(params.FOO)",
 					}},
 				},
 			},
@@ -153,10 +155,10 @@ var (
 			Name: "some-secret",
 			VolumeSource: corev1.VolumeSource{
 				Secret: &corev1.SecretVolumeSource{
-					SecretName: "$(inputs.params.FOO)",
+					SecretName: "$(params.FOO)",
 					Items: []corev1.KeyToPath{{
-						Key:  "$(inputs.params.FOO)",
-						Path: "$(inputs.params.FOO)",
+						Key:  "$(params.FOO)",
+						Path: "$(params.FOO)",
 					}},
 				},
 			},
@@ -164,7 +166,7 @@ var (
 			Name: "some-pvc",
 			VolumeSource: corev1.VolumeSource{
 				PersistentVolumeClaim: &corev1.PersistentVolumeClaimVolumeSource{
-					ClaimName: "$(inputs.params.FOO)",
+					ClaimName: "$(params.FOO)",
 				},
 			},
 		}, {
@@ -174,16 +176,16 @@ var (
 					Sources: []corev1.VolumeProjection{{
 						ConfigMap: &corev1.ConfigMapProjection{
 							LocalObjectReference: corev1.LocalObjectReference{
-								Name: "$(inputs.params.FOO)",
+								Name: "$(params.FOO)",
 							},
 						},
 						Secret: &corev1.SecretProjection{
 							LocalObjectReference: corev1.LocalObjectReference{
-								Name: "$(inputs.params.FOO)",
+								Name: "$(params.FOO)",
 							},
 						},
 						ServiceAccountToken: &corev1.ServiceAccountTokenProjection{
-							Audience: "$(inputs.params.FOO)",
+							Audience: "$(params.FOO)",
 						},
 					}},
 				},
@@ -193,10 +195,10 @@ var (
 			VolumeSource: corev1.VolumeSource{
 				CSI: &corev1.CSIVolumeSource{
 					VolumeAttributes: map[string]string{
-						"secretProviderClass": "$(inputs.params.FOO)",
+						"secretProviderClass": "$(params.FOO)",
 					},
 					NodePublishSecretRef: &corev1.LocalObjectReference{
-						Name: "$(inputs.params.FOO)",
+						Name: "$(params.FOO)",
 					},
 				},
 			},
@@ -244,7 +246,7 @@ var (
 			Name:    "image-with-c-specified",
 			Image:   "some-other-image",
 			Command: []string{"echo"},
-			Args:    []string{"first", "second", "$(inputs.params.array-param)", "last"},
+			Args:    []string{"first", "second", "$(params.array-param)", "last"},
 		}}},
 	}
 
@@ -256,7 +258,7 @@ var (
 			Name:    "image-with-c-specified",
 			Image:   "some-other-image",
 			Command: []string{"echo"},
-			Args:    []string{"$(inputs.params.normal-param)", "second", "$(inputs.params.array-param)", "last"},
+			Args:    []string{"$(params.normal-param)", "second", "$(params.array-param)", "last"},
 		}}},
 	}
 
@@ -267,20 +269,20 @@ var (
 		}}, {Container: corev1.Container{
 			Name:    "image-with-c-specified",
 			Image:   "some-other-image",
-			Command: []string{"cmd", "$(inputs.params.another-array-param)"},
-			Args:    []string{"first", "second", "$(inputs.params.array-param)", "last"},
+			Command: []string{"cmd", "$(params.another-array-param)"},
+			Args:    []string{"first", "second", "$(params.array-param)", "last"},
 		}}},
 	}
 
 	multipleArrayAndStringsParamsTaskSpec = &v1beta1.TaskSpec{
 		Steps: []v1beta1.Step{{Container: corev1.Container{
 			Name:  "simple-image",
-			Image: "image-$(inputs.params.string-param2)",
+			Image: "image-$(params.string-param2)",
 		}}, {Container: corev1.Container{
 			Name:    "image-with-c-specified",
 			Image:   "some-other-image",
-			Command: []string{"cmd", "$(inputs.params.array-param1)"},
-			Args:    []string{"$(inputs.params.array-param2)", "second", "$(inputs.params.array-param1)", "$(inputs.params.string-param1)", "last"},
+			Command: []string{"cmd", "$(params.array-param1)"},
+			Args:    []string{"$(params.array-param2)", "second", "$(params.array-param1)", "$(params.string-param1)", "last"},
 		}}},
 	}
 
@@ -874,16 +876,14 @@ func TestApplyWorkspaces_IsolatedWorkspaces(t *testing.T) {
 func TestContext(t *testing.T) {
 	for _, tc := range []struct {
 		description string
-		rtr         resources.ResolvedTaskResources
+		taskName    string
 		tr          v1beta1.TaskRun
 		spec        v1beta1.TaskSpec
 		want        v1beta1.TaskSpec
 	}{{
 		description: "context taskName replacement without taskRun in spec container",
-		rtr: resources.ResolvedTaskResources{
-			TaskName: "Task1",
-		},
-		tr: v1beta1.TaskRun{},
+		taskName:    "Task1",
+		tr:          v1beta1.TaskRun{},
 		spec: v1beta1.TaskSpec{
 			Steps: []v1beta1.Step{{
 				Container: corev1.Container{
@@ -902,9 +902,7 @@ func TestContext(t *testing.T) {
 		},
 	}, {
 		description: "context taskName replacement with taskRun in spec container",
-		rtr: resources.ResolvedTaskResources{
-			TaskName: "Task1",
-		},
+		taskName:    "Task1",
 		tr: v1beta1.TaskRun{
 			ObjectMeta: metav1.ObjectMeta{
 				Name: "taskrunName",
@@ -928,9 +926,7 @@ func TestContext(t *testing.T) {
 		},
 	}, {
 		description: "context taskRunName replacement with defined taskRun in spec container",
-		rtr: resources.ResolvedTaskResources{
-			TaskName: "Task1",
-		},
+		taskName:    "Task1",
 		tr: v1beta1.TaskRun{
 			ObjectMeta: metav1.ObjectMeta{
 				Name: "taskrunName",
@@ -954,10 +950,8 @@ func TestContext(t *testing.T) {
 		},
 	}, {
 		description: "context taskRunName replacement with no defined taskRun name in spec container",
-		rtr: resources.ResolvedTaskResources{
-			TaskName: "Task1",
-		},
-		tr: v1beta1.TaskRun{},
+		taskName:    "Task1",
+		tr:          v1beta1.TaskRun{},
 		spec: v1beta1.TaskSpec{
 			Steps: []v1beta1.Step{{
 				Container: corev1.Container{
@@ -976,10 +970,8 @@ func TestContext(t *testing.T) {
 		},
 	}, {
 		description: "context taskRun namespace replacement with no defined namepsace in spec container",
-		rtr: resources.ResolvedTaskResources{
-			TaskName: "Task1",
-		},
-		tr: v1beta1.TaskRun{},
+		taskName:    "Task1",
+		tr:          v1beta1.TaskRun{},
 		spec: v1beta1.TaskSpec{
 			Steps: []v1beta1.Step{{
 				Container: corev1.Container{
@@ -998,9 +990,7 @@ func TestContext(t *testing.T) {
 		},
 	}, {
 		description: "context taskRun namespace replacement with defined namepsace in spec container",
-		rtr: resources.ResolvedTaskResources{
-			TaskName: "Task1",
-		},
+		taskName:    "Task1",
 		tr: v1beta1.TaskRun{
 			ObjectMeta: metav1.ObjectMeta{
 				Name:      "taskrunName",
@@ -1025,7 +1015,6 @@ func TestContext(t *testing.T) {
 		},
 	}, {
 		description: "context taskRunName replacement with no defined taskName in spec container",
-		rtr:         resources.ResolvedTaskResources{},
 		tr:          v1beta1.TaskRun{},
 		spec: v1beta1.TaskSpec{
 			Steps: []v1beta1.Step{{
@@ -1045,9 +1034,7 @@ func TestContext(t *testing.T) {
 		},
 	}, {
 		description: "context UID replacement",
-		rtr: resources.ResolvedTaskResources{
-			TaskName: "Task1",
-		},
+		taskName:    "Task1",
 		tr: v1beta1.TaskRun{
 			ObjectMeta: metav1.ObjectMeta{
 				UID: "UID-1",
@@ -1069,9 +1056,67 @@ func TestContext(t *testing.T) {
 				},
 			}},
 		},
+	}, {
+		description: "context retry count replacement",
+		tr: v1beta1.TaskRun{
+			Status: v1beta1.TaskRunStatus{
+				TaskRunStatusFields: v1beta1.TaskRunStatusFields{
+					RetriesStatus: []v1beta1.TaskRunStatus{{
+						Status: duckv1beta1.Status{
+							Conditions: []apis.Condition{{
+								Type:   apis.ConditionSucceeded,
+								Status: corev1.ConditionFalse,
+							}},
+						},
+					}, {
+						Status: duckv1beta1.Status{
+							Conditions: []apis.Condition{{
+								Type:   apis.ConditionSucceeded,
+								Status: corev1.ConditionFalse,
+							}},
+						},
+					}},
+				},
+			},
+		},
+		spec: v1beta1.TaskSpec{
+			Steps: []v1beta1.Step{{
+				Container: corev1.Container{
+					Name:  "ImageName",
+					Image: "$(context.task.retry-count)-1",
+				},
+			}},
+		},
+		want: v1beta1.TaskSpec{
+			Steps: []v1beta1.Step{{
+				Container: corev1.Container{
+					Name:  "ImageName",
+					Image: "2-1",
+				},
+			}},
+		},
+	}, {
+		description: "context retry count replacement with task that never retries",
+		tr:          v1beta1.TaskRun{},
+		spec: v1beta1.TaskSpec{
+			Steps: []v1beta1.Step{{
+				Container: corev1.Container{
+					Name:  "ImageName",
+					Image: "$(context.task.retry-count)-1",
+				},
+			}},
+		},
+		want: v1beta1.TaskSpec{
+			Steps: []v1beta1.Step{{
+				Container: corev1.Container{
+					Name:  "ImageName",
+					Image: "0-1",
+				},
+			}},
+		},
 	}} {
 		t.Run(tc.description, func(t *testing.T) {
-			got := resources.ApplyContexts(&tc.spec, &tc.rtr, &tc.tr)
+			got := resources.ApplyContexts(&tc.spec, tc.taskName, &tc.tr)
 			if d := cmp.Diff(&tc.want, got); d != "" {
 				t.Errorf(diff.PrintWantGot(d))
 			}
@@ -1083,7 +1128,7 @@ func TestTaskResults(t *testing.T) {
 	names.TestingSeed()
 	ts := &v1beta1.TaskSpec{
 		Results: []v1beta1.TaskResult{{
-			Name:        "current-date-unix-timestamp",
+			Name:        "current.date.unix.timestamp",
 			Description: "The current date in unix timestamp format",
 		}, {
 			Name:        "current-date-human-readable",
@@ -1093,21 +1138,28 @@ func TestTaskResults(t *testing.T) {
 			Container: corev1.Container{
 				Name:  "print-date-unix-timestamp",
 				Image: "bash:latest",
-				Args:  []string{"$(results.current-date-unix-timestamp.path)"},
+				Args:  []string{"$(results[\"current.date.unix.timestamp\"].path)"},
 			},
-			Script: "#!/usr/bin/env bash\ndate +%s | tee $(results.current-date-unix-timestamp.path)",
+			Script: "#!/usr/bin/env bash\ndate +%s | tee $(results[\"current.date.unix.timestamp\"].path)",
 		}, {
 			Container: corev1.Container{
 				Name:  "print-date-human-readable",
 				Image: "bash:latest",
 			},
 			Script: "#!/usr/bin/env bash\ndate | tee $(results.current-date-human-readable.path)",
+		}, {
+			Container: corev1.Container{
+				Name:  "print-date-human-readable-again",
+				Image: "bash:latest",
+			},
+			Script: "#!/usr/bin/env bash\ndate | tee $(results['current-date-human-readable'].path)",
 		}},
 	}
 	want := applyMutation(ts, func(spec *v1beta1.TaskSpec) {
-		spec.Steps[0].Script = "#!/usr/bin/env bash\ndate +%s | tee /tekton/results/current-date-unix-timestamp"
-		spec.Steps[0].Args[0] = "/tekton/results/current-date-unix-timestamp"
+		spec.Steps[0].Script = "#!/usr/bin/env bash\ndate +%s | tee /tekton/results/current.date.unix.timestamp"
+		spec.Steps[0].Args[0] = "/tekton/results/current.date.unix.timestamp"
 		spec.Steps[1].Script = "#!/usr/bin/env bash\ndate | tee /tekton/results/current-date-human-readable"
+		spec.Steps[2].Script = "#!/usr/bin/env bash\ndate | tee /tekton/results/current-date-human-readable"
 	})
 	got := resources.ApplyTaskResults(ts)
 	if d := cmp.Diff(want, got); d != "" {
@@ -1115,6 +1167,38 @@ func TestTaskResults(t *testing.T) {
 	}
 }
 
+func TestApplyStepExitCodePath(t *testing.T) {
+	names.TestingSeed()
+	ts := &v1beta1.TaskSpec{
+		Steps: []v1beta1.Step{{
+			Container: corev1.Container{
+				Image: "bash:latest",
+			},
+			Script: "#!/usr/bin/env bash\nexit 11",
+		}, {
+			Container: corev1.Container{
+				Name:  "failing-step",
+				Image: "bash:latest",
+			},
+			Script: "#!/usr/bin/env bash\ncat $(steps.step-unnamed-0.exitCode.path)",
+		}, {
+			Container: corev1.Container{
+				Name:  "check-failing-step",
+				Image: "bash:latest",
+			},
+			Script: "#!/usr/bin/env bash\ncat $(steps.step-failing-step.exitCode.path)",
+		}},
+	}
+	expected := applyMutation(ts, func(spec *v1beta1.TaskSpec) {
+		spec.Steps[1].Script = "#!/usr/bin/env bash\ncat /tekton/steps/step-unnamed-0/exitCode"
+		spec.Steps[2].Script = "#!/usr/bin/env bash\ncat /tekton/steps/step-failing-step/exitCode"
+	})
+	got := resources.ApplyStepExitCodePath(ts)
+	if d := cmp.Diff(expected, got); d != "" {
+		t.Errorf("ApplyStepExitCodePath() got diff %s", diff.PrintWantGot(d))
+	}
+}
+
 func TestApplyCredentialsPath(t *testing.T) {
 	for _, tc := range []struct {
 		description string
