@@ -1,7 +1,8 @@
+//go:build e2e
 // +build e2e
 
 /*
-Copyright 2019 The Tekton Authors
+Copyright 2021 The Tekton Authors
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
@@ -25,6 +26,8 @@ import (
 	"testing"
 	"time"
 
+	"github.com/tektoncd/pipeline/test/parse"
+
 	"github.com/google/go-cmp/cmp"
 	"github.com/tektoncd/pipeline/pkg/apis/pipeline/v1beta1"
 	corev1 "k8s.io/api/core/v1"
@@ -48,36 +51,39 @@ func TestPipelineRunTimeout(t *testing.T) {
 	defer tearDown(context.Background(), t, c, namespace)
 
 	t.Logf("Creating Task in namespace %s", namespace)
-	task := &v1beta1.Task{
-		ObjectMeta: metav1.ObjectMeta{Name: helpers.ObjectNameForTest(t), Namespace: namespace},
-		Spec: v1beta1.TaskSpec{
-			Steps: []v1beta1.Step{{Container: corev1.Container{
-				Image:   "mirror.gcr.io/library/busybox",
-				Command: []string{"/bin/sh"},
-				Args:    []string{"-c", "sleep 10"},
-			}}},
-		},
-	}
+	task := parse.MustParseTask(t, fmt.Sprintf(`
+metadata:
+  name: %s
+  namespace: %s
+spec:
+  steps:
+  - image: busybox
+    command: ['/bin/sh']
+    args: ['-c', 'sleep 10']
+`, helpers.ObjectNameForTest(t), namespace))
 	if _, err := c.TaskClient.Create(ctx, task, metav1.CreateOptions{}); err != nil {
 		t.Fatalf("Failed to create Task `%s`: %s", task.Name, err)
 	}
 
-	pipeline := &v1beta1.Pipeline{
-		ObjectMeta: metav1.ObjectMeta{Name: helpers.ObjectNameForTest(t), Namespace: namespace},
-		Spec: v1beta1.PipelineSpec{
-			Tasks: []v1beta1.PipelineTask{{
-				Name:    "foo",
-				TaskRef: &v1beta1.TaskRef{Name: task.Name},
-			}},
-		},
-	}
-	pipelineRun := &v1beta1.PipelineRun{
-		ObjectMeta: metav1.ObjectMeta{Name: helpers.ObjectNameForTest(t), Namespace: namespace},
-		Spec: v1beta1.PipelineRunSpec{
-			PipelineRef: &v1beta1.PipelineRef{Name: pipeline.Name},
-			Timeout:     &metav1.Duration{Duration: 5 * time.Second},
-		},
-	}
+	pipeline := parse.MustParsePipeline(t, fmt.Sprintf(`
+metadata:
+  name: %s
+  namespace: %s
+spec:
+  tasks:
+  - name: foo
+    taskRef:
+      name: %s
+`, helpers.ObjectNameForTest(t), namespace, task.Name))
+	pipelineRun := parse.MustParsePipelineRun(t, fmt.Sprintf(`
+metadata:
+  name: %s
+  namespace: %s
+spec:
+  pipelineRef:
+    name: %s
+  timeout: 5s
+`, helpers.ObjectNameForTest(t), namespace, pipeline.Name))
 	if _, err := c.PipelineClient.Create(ctx, pipeline, metav1.CreateOptions{}); err != nil {
 		t.Fatalf("Failed to create Pipeline `%s`: %s", pipeline.Name, err)
 	}
@@ -141,21 +147,24 @@ func TestPipelineRunTimeout(t *testing.T) {
 
 	// Verify that we can create a second Pipeline using the same Task without a Pipeline-level timeout that will not
 	// time out
-	secondPipeline := &v1beta1.Pipeline{
-		ObjectMeta: metav1.ObjectMeta{Name: helpers.ObjectNameForTest(t), Namespace: namespace},
-		Spec: v1beta1.PipelineSpec{
-			Tasks: []v1beta1.PipelineTask{{
-				Name:    "foo",
-				TaskRef: &v1beta1.TaskRef{Name: task.Name},
-			}},
-		},
-	}
-	secondPipelineRun := &v1beta1.PipelineRun{
-		ObjectMeta: metav1.ObjectMeta{Name: helpers.ObjectNameForTest(t), Namespace: namespace},
-		Spec: v1beta1.PipelineRunSpec{
-			PipelineRef: &v1beta1.PipelineRef{Name: secondPipeline.Name},
-		},
-	}
+	secondPipeline := parse.MustParsePipeline(t, fmt.Sprintf(`
+metadata:
+  name: %s
+  namespace: %s
+spec:
+  tasks:
+  - name: foo
+    taskRef:
+      name: %s
+`, helpers.ObjectNameForTest(t), namespace, task.Name))
+	secondPipelineRun := parse.MustParsePipelineRun(t, fmt.Sprintf(`
+metadata:
+  name: %s
+  namespace: %s
+spec:
+  pipelineRef:
+    name: %s
+`, helpers.ObjectNameForTest(t), namespace, secondPipeline.Name))
 	if _, err := c.PipelineClient.Create(ctx, secondPipeline, metav1.CreateOptions{}); err != nil {
 		t.Fatalf("Failed to create Pipeline `%s`: %s", secondPipeline.Name, err)
 	}
@@ -181,34 +190,25 @@ func TestStepTimeout(t *testing.T) {
 
 	t.Logf("Creating Task with Step step-no-timeout, Step step-timeout, and Step step-canceled in namespace %s", namespace)
 
-	taskRun := &v1beta1.TaskRun{
-		ObjectMeta: metav1.ObjectMeta{Name: helpers.ObjectNameForTest(t), Namespace: namespace},
-		Spec: v1beta1.TaskRunSpec{
-			TaskSpec: &v1beta1.TaskSpec{
-				Steps: []v1beta1.Step{{
-					Container: corev1.Container{
-						Name:  "no-timeout",
-						Image: "mirror.gcr.io/library/busybox",
-					},
-					Script:  "sleep 1",
-					Timeout: &metav1.Duration{Duration: 2 * time.Second},
-				}, {
-					Container: corev1.Container{
-						Name:  "timeout",
-						Image: "mirror.gcr.io/library/busybox",
-					},
-					Script:  "sleep 1",
-					Timeout: &metav1.Duration{Duration: time.Millisecond},
-				}, {
-					Container: corev1.Container{
-						Name:  "canceled",
-						Image: "mirror.gcr.io/library/busybox",
-					},
-					Script: "sleep 1",
-				}},
-			},
-		},
-	}
+	taskRun := parse.MustParseTaskRun(t, fmt.Sprintf(`
+metadata:
+  name: %s
+  namespace: %s
+spec:
+  taskSpec:
+    steps:
+    - name: no-timeout
+      image: busybox
+      script: sleep 1
+      timeout: 2s
+    - name: timeout
+      image: busybox
+      script: sleep 1
+      timeout: 1ms
+    - name: canceled
+      image: busybox
+      script: sleep 1
+`, helpers.ObjectNameForTest(t), namespace))
 	t.Logf("Creating TaskRun %s in namespace %s", taskRun.Name, namespace)
 	if _, err := c.TaskRunClient.Create(ctx, taskRun, metav1.CreateOptions{}); err != nil {
 		t.Fatalf("Failed to create TaskRun `%s`: %s", taskRun.Name, err)
@@ -238,10 +238,49 @@ func TestStepTimeout(t *testing.T) {
 
 }
 
+// TestStepTimeoutWithWS is an integration test that will verify a Step can be timed out.
+func TestStepTimeoutWithWS(t *testing.T) {
+	t.Parallel()
+	ctx, cancel := context.WithTimeout(context.Background(), 2*time.Minute)
+	defer cancel()
+	c, namespace := setup(ctx, t)
+
+	knativetest.CleanupOnInterrupt(func() { tearDown(context.Background(), t, c, namespace) }, t.Logf)
+	defer tearDown(context.Background(), t, c, namespace)
+
+	taskRun := parse.MustParseTaskRun(t, `
+metadata:
+  name: taskrun-with-timeout-step
+spec:
+  workspaces:
+    - name: test
+      emptyDir: {}
+  taskSpec:
+    workspaces:
+      - name: test
+    steps:
+      - name: timeout
+        image: busybox
+        script: sleep 1
+        timeout: 1ms`)
+
+	t.Logf("Creating TaskRun %s in namespace %s", taskRun.Name, namespace)
+	if _, err := c.TaskRunClient.Create(ctx, taskRun, metav1.CreateOptions{}); err != nil {
+		t.Fatalf("Failed to create TaskRun `%s`: %s", taskRun.Name, err)
+	}
+
+	failMsg := "\"step-timeout\" exited because the step exceeded the specified timeout limit"
+	t.Logf("Waiting for %s in namespace %s to time out", "step-timeout", namespace)
+	if err := WaitForTaskRunState(ctx, c, taskRun.Name, FailedWithMessage(failMsg, taskRun.Name), "StepTimeout"); err != nil {
+		t.Logf("Error in taskRun %s status: %s\n", taskRun.Name, err)
+		t.Errorf("Expected: %s", failMsg)
+	}
+}
+
 // TestTaskRunTimeout is an integration test that will verify a TaskRun can be timed out.
 func TestTaskRunTimeout(t *testing.T) {
 	t.Parallel()
-	timeout := 30 * time.Second
+	timeout := 1 * time.Second
 	ctx, cancel := context.WithTimeout(context.Background(), timeout+2*time.Minute)
 	defer cancel()
 	c, namespace := setup(ctx, t)
@@ -250,28 +289,28 @@ func TestTaskRunTimeout(t *testing.T) {
 	defer tearDown(context.Background(), t, c, namespace)
 
 	t.Logf("Creating Task and TaskRun in namespace %s", namespace)
-	task := &v1beta1.Task{
-		ObjectMeta: metav1.ObjectMeta{Name: helpers.ObjectNameForTest(t), Namespace: namespace},
-		Spec: v1beta1.TaskSpec{
-			Steps: []v1beta1.Step{{Container: corev1.Container{
-				Image:   "mirror.gcr.io/library/busybox",
-				Command: []string{"/bin/sh"},
-				Args:    []string{"-c", "sleep 3000"},
-			}}},
-		},
-	}
+	task := parse.MustParseTask(t, fmt.Sprintf(`
+metadata:
+  name: %s
+  namespace: %s
+spec:
+  steps:
+  - image: busybox
+    command: ['/bin/sh']
+    args: ['-c', 'sleep 3000']
+`, helpers.ObjectNameForTest(t), namespace))
 	if _, err := c.TaskClient.Create(ctx, task, metav1.CreateOptions{}); err != nil {
 		t.Fatalf("Failed to create Task `%s`: %s", task.Name, err)
 	}
-	taskRun := &v1beta1.TaskRun{
-		ObjectMeta: metav1.ObjectMeta{Name: helpers.ObjectNameForTest(t), Namespace: namespace},
-		Spec: v1beta1.TaskRunSpec{
-			TaskRef: &v1beta1.TaskRef{Name: task.Name},
-			// Do not reduce this timeout. Taskrun e2e test is also verifying
-			// if reconcile is triggered from timeout handler and not by pod informers
-			Timeout: &metav1.Duration{Duration: timeout},
-		},
-	}
+	taskRun := parse.MustParseTaskRun(t, fmt.Sprintf(`
+metadata:
+  name: %s
+  namespace: %s
+spec:
+  taskRef:
+    name: %s
+  timeout: %s
+`, helpers.ObjectNameForTest(t), namespace, task.Name, timeout))
 	if _, err := c.TaskRunClient.Create(ctx, taskRun, metav1.CreateOptions{}); err != nil {
 		t.Fatalf("Failed to create TaskRun `%s`: %s", taskRun.Name, err)
 	}
@@ -306,26 +345,26 @@ func TestPipelineTaskTimeout(t *testing.T) {
 	defer tearDown(context.Background(), t, c, namespace)
 
 	t.Logf("Creating Tasks in namespace %s", namespace)
-	task1 := &v1beta1.Task{
-		ObjectMeta: metav1.ObjectMeta{Name: helpers.ObjectNameForTest(t), Namespace: namespace},
-		Spec: v1beta1.TaskSpec{
-			Steps: []v1beta1.Step{{Container: corev1.Container{
-				Image:   "mirror.gcr.io/library/busybox",
-				Command: []string{"sleep"},
-				Args:    []string{"1s"},
-			}}},
-		},
-	}
-	task2 := &v1beta1.Task{
-		ObjectMeta: metav1.ObjectMeta{Name: helpers.ObjectNameForTest(t), Namespace: namespace},
-		Spec: v1beta1.TaskSpec{
-			Steps: []v1beta1.Step{{Container: corev1.Container{
-				Image:   "mirror.gcr.io/library/busybox",
-				Command: []string{"sleep"},
-				Args:    []string{"10s"},
-			}}},
-		},
-	}
+	task1 := parse.MustParseTask(t, fmt.Sprintf(`
+metadata:
+  name: %s
+  namespace: %s
+spec:
+  steps:
+  - image: busybox
+    command: ['/bin/sh']
+    args: ['-c', 'sleep 1s']
+`, helpers.ObjectNameForTest(t), namespace))
+	task2 := parse.MustParseTask(t, fmt.Sprintf(`
+metadata:
+  name: %s
+  namespace: %s
+spec:
+  steps:
+  - image: busybox
+    command: ['/bin/sh']
+    args: ['-c', 'sleep 10s']
+`, helpers.ObjectNameForTest(t), namespace))
 
 	if _, err := c.TaskClient.Create(ctx, task1, metav1.CreateOptions{}); err != nil {
 		t.Fatalf("Failed to create Task `%s`: %s", task1.Name, err)
@@ -334,26 +373,29 @@ func TestPipelineTaskTimeout(t *testing.T) {
 		t.Fatalf("Failed to create Task `%s`: %s", task2.Name, err)
 	}
 
-	pipeline := &v1beta1.Pipeline{
-		ObjectMeta: metav1.ObjectMeta{Name: helpers.ObjectNameForTest(t), Namespace: namespace},
-		Spec: v1beta1.PipelineSpec{
-			Tasks: []v1beta1.PipelineTask{{
-				Name:    "pipelinetask1",
-				TaskRef: &v1beta1.TaskRef{Name: task1.Name},
-				Timeout: &metav1.Duration{Duration: 60 * time.Second},
-			}, {
-				Name:    "pipelinetask2",
-				TaskRef: &v1beta1.TaskRef{Name: task2.Name},
-				Timeout: &metav1.Duration{Duration: 5 * time.Second},
-			}},
-		},
-	}
-	pipelineRun := &v1beta1.PipelineRun{
-		ObjectMeta: metav1.ObjectMeta{Name: helpers.ObjectNameForTest(t), Namespace: namespace},
-		Spec: v1beta1.PipelineRunSpec{
-			PipelineRef: &v1beta1.PipelineRef{Name: pipeline.Name},
-		},
-	}
+	pipeline := parse.MustParsePipeline(t, fmt.Sprintf(`
+metadata:
+  name: %s
+  namespace: %s
+spec:
+  tasks:
+  - name: pipelinetask1
+    taskRef:
+      name: %s
+    timeout: 60s
+  - name: pipelinetask2
+    taskRef:
+      name: %s
+    timeout: 5s
+`, helpers.ObjectNameForTest(t), namespace, task1.Name, task2.Name))
+	pipelineRun := parse.MustParsePipelineRun(t, fmt.Sprintf(`
+metadata:
+  name: %s
+  namespace: %s
+spec:
+  pipelineRef:
+    name: %s
+`, helpers.ObjectNameForTest(t), namespace, pipeline.Name))
 
 	if _, err := c.PipelineClient.Create(ctx, pipeline, metav1.CreateOptions{}); err != nil {
 		t.Fatalf("Failed to create Pipeline `%s`: %s", pipeline.Name, err)
@@ -435,3 +477,155 @@ func TestPipelineTaskTimeout(t *testing.T) {
 	}
 	wg.Wait()
 }
+
+// TestPipelineRunTasksTimeout is an integration test that will
+// verify that pipelinerun tasksTimeout works and leads to the the correct PipelineRun and TaskRun statuses
+// and pod deletions.
+func TestPipelineRunTasksTimeout(t *testing.T) {
+	t.Parallel()
+	// cancel the context after we have waited a suitable buffer beyond the given deadline.
+	ctx, cancel := context.WithTimeout(context.Background(), timeout+2*time.Minute)
+	defer cancel()
+	c, namespace := setup(ctx, t, requireAnyGate(map[string]string{"enable-api-fields": "alpha"}))
+
+	knativetest.CleanupOnInterrupt(func() { tearDown(context.Background(), t, c, namespace) }, t.Logf)
+	defer tearDown(context.Background(), t, c, namespace)
+
+	t.Logf("Creating Task in namespace %s", namespace)
+	task := parse.MustParseTask(t, fmt.Sprintf(`
+metadata:
+  name: %s
+  namespace: %s
+spec:
+  steps:
+  - image: busybox
+    command: ['/bin/sh']
+    args: ['-c', 'sleep 30']
+`, helpers.ObjectNameForTest(t), namespace))
+	if _, err := c.TaskClient.Create(ctx, task, metav1.CreateOptions{}); err != nil {
+		t.Fatalf("Failed to create Task `%s`: %s", task.Name, err)
+	}
+
+	t.Logf("Creating Finally Task in namespace %s", namespace)
+	fTask := parse.MustParseTask(t, fmt.Sprintf(`
+metadata:
+  name: %s
+  namespace: %s
+spec:
+  steps:
+  - image: busybox
+    command: ['/bin/sh']
+    args: ['-c', 'sleep 1']
+`, helpers.ObjectNameForTest(t), namespace))
+	if _, err := c.TaskClient.Create(ctx, fTask, metav1.CreateOptions{}); err != nil {
+		t.Fatalf("Failed to create Task `%s`: %s", fTask.Name, err)
+	}
+
+	pipeline := parse.MustParsePipeline(t, fmt.Sprintf(`
+metadata:
+  name: %s
+  namespace: %s
+spec:
+  tasks:
+  - name: dagtask
+    taskRef:
+      name: %s
+  finally:
+  - name: finallytask
+    taskRef:
+      name: %s
+`, helpers.ObjectNameForTest(t), namespace, task.Name, fTask.Name))
+	pipelineRun := parse.MustParsePipelineRun(t, fmt.Sprintf(`
+metadata:
+  name: %s
+  namespace: %s
+spec:
+  pipelineRef:
+    name: %s
+  timeouts:
+    pipeline: 60s
+    tasks: 20s
+    finally: 20s
+`, helpers.ObjectNameForTest(t), namespace, pipeline.Name))
+	if _, err := c.PipelineClient.Create(ctx, pipeline, metav1.CreateOptions{}); err != nil {
+		t.Fatalf("Failed to create Pipeline `%s`: %s", pipeline.Name, err)
+	}
+	if _, err := c.PipelineRunClient.Create(ctx, pipelineRun, metav1.CreateOptions{}); err != nil {
+		t.Fatalf("Failed to create PipelineRun `%s`: %s", pipelineRun.Name, err)
+	}
+
+	t.Logf("Waiting for Pipelinerun %s in namespace %s to be started", pipelineRun.Name, namespace)
+	if err := WaitForPipelineRunState(ctx, c, pipelineRun.Name, timeout, Running(pipelineRun.Name), "PipelineRunRunning"); err != nil {
+		t.Fatalf("Error waiting for PipelineRun %s to be running: %s", pipelineRun.Name, err)
+	}
+
+	taskrunList, err := c.TaskRunClient.List(ctx, metav1.ListOptions{LabelSelector: fmt.Sprintf("tekton.dev/pipelineRun=%s", pipelineRun.Name)})
+	if err != nil {
+		t.Fatalf("Error listing TaskRuns for PipelineRun %s: %s", pipelineRun.Name, err)
+	}
+
+	t.Logf("Waiting for TaskRuns from PipelineRun %s in namespace %s to be running", pipelineRun.Name, namespace)
+	errChan := make(chan error, len(taskrunList.Items))
+	defer close(errChan)
+
+	for _, taskrunItem := range taskrunList.Items {
+		go func(name string) {
+			err := WaitForTaskRunState(ctx, c, name, Running(name), "TaskRunRunning")
+			errChan <- err
+		}(taskrunItem.Name)
+	}
+
+	for i := 1; i <= len(taskrunList.Items); i++ {
+		if err := <-errChan; err != nil {
+			t.Errorf("Error waiting for TaskRun %s to be running: %v", taskrunList.Items[i-1].Name, err)
+		}
+	}
+
+	if _, err := c.PipelineRunClient.Get(ctx, pipelineRun.Name, metav1.GetOptions{}); err != nil {
+		t.Fatalf("Failed to get PipelineRun `%s`: %s", pipelineRun.Name, err)
+	}
+
+	t.Logf("Waiting for PipelineRun %s in namespace %s to be failed", pipelineRun.Name, namespace)
+	if err := WaitForPipelineRunState(ctx, c, pipelineRun.Name, timeout, FailedWithReason(v1beta1.PipelineRunReasonFailed.String(), pipelineRun.Name), "PipelineRunFailed"); err != nil {
+		t.Errorf("Error waiting for PipelineRun %s to finish: %s", pipelineRun.Name, err)
+	}
+
+	t.Logf("Waiting for TaskRun from PipelineRun %s in namespace %s to time out and finally TaskRun to be successful", pipelineRun.Name, namespace)
+	var wg sync.WaitGroup
+	for _, taskrunItem := range taskrunList.Items {
+		wg.Add(1)
+		go func(tr v1beta1.TaskRun) {
+			defer wg.Done()
+			name := tr.Name
+			err := WaitForTaskRunState(ctx, c, name, func(ca apis.ConditionAccessor) (bool, error) {
+				cond := ca.GetCondition(apis.ConditionSucceeded)
+				if cond != nil {
+					if tr.Spec.TaskRef.Name == fTask.Name && cond.Status == corev1.ConditionTrue {
+						if cond.Reason == "Succeeded" {
+							return true, nil
+						}
+						return true, fmt.Errorf("taskRun %q completed with the wrong reason: %s", fTask.Name, cond.Reason)
+					} else if tr.Spec.TaskRef.Name == fTask.Name && cond.Status == corev1.ConditionFalse {
+						return true, fmt.Errorf("taskRun %q failed, but should have been Succeeded", name)
+					}
+
+					if tr.Spec.TaskRef.Name == task.Name && cond.Status == corev1.ConditionFalse {
+						if cond.Reason == "TaskRunTimeout" {
+							return true, nil
+						}
+						return true, fmt.Errorf("taskRun %q completed with the wrong reason: %s", task.Name, cond.Reason)
+					} else if tr.Spec.TaskRef.Name == task.Name && cond.Status == corev1.ConditionTrue {
+						return true, fmt.Errorf("taskRun %q should have timed out", name)
+					}
+				}
+				return false, nil
+			}, "TaskRunTimeout")
+
+			if err != nil {
+				t.Errorf("Error waiting for TaskRun %s to timeout: %s", name, err)
+			}
+
+		}(taskrunItem)
+	}
+	wg.Wait()
+}
